@@ -2,6 +2,9 @@ import argparse
 import importlib
 import importlib.util
 import os
+import re
+import sys
+import threading
 from collections import namedtuple
 from contextlib import contextmanager
 
@@ -945,7 +948,7 @@ class BwaIndexer(object):
             )
 
         self.algo_type = algo_map[algorithm]
-        
+
         # Progress tracking
         self.progress = {
             "status": "idle",
@@ -962,10 +965,6 @@ class BwaIndexer(object):
         :param prefix: Output prefix for index files (default: same as FASTA file)
         :returns: Path to the index prefix
         """
-        import os
-        import sys
-        import threading
-
         if not os.path.exists(fasta_file):
             raise FileNotFoundError(f"FASTA file not found: {fasta_file}")
 
@@ -990,37 +989,32 @@ class BwaIndexer(object):
         libbwa.bwa_verbose = int(self.verbose)
 
         if self.capture_progress:
-            # Use os.pipe() to capture stderr from C code
-            # Threading is necessary because:
-            # 1. C code writes to stderr (file descriptor 2)
-            # 2. Pipe buffers are limited (~64KB)
-            # 3. If we don't read continuously, the pipe fills and C code blocks
-            # 4. We can't read after C completes because data is streamed
             read_fd, write_fd = os.pipe()
             old_stderr = os.dup(2)
             os.dup2(write_fd, 2)
-            os.close(write_fd)
-            
+
             def reader():
                 """Read stderr in separate thread to prevent pipe deadlock."""
-                with os.fdopen(read_fd, 'r', errors='replace') as f:
+                with os.fdopen(read_fd, "r", errors="replace") as f:
                     for line in f:
                         line = line.rstrip()
                         if line:
                             self._parse_progress_line(line)
-            
+
             reader_thread = threading.Thread(target=reader, daemon=True)
             reader_thread.start()
-            
+
             try:
                 result = libbwa.bwa_idx_build(
                     fasta_bytes, prefix_bytes, self.algo_type, self.block_size
                 )
             finally:
                 sys.stderr.flush()
-                os.dup2(old_stderr, 2)
-                os.close(old_stderr)
-                reader_thread.join(timeout=1)
+                temp_stderr = os.dup(2)  # Save the current FD 2
+                os.dup2(old_stderr, 2)  # Restore original stderr NOW
+                os.close(temp_stderr)  # Close the pipe write end to signal EOF
+                os.close(old_stderr)  # Close the saved original stderr
+                reader_thread.join(timeout=5)
         else:
             result = libbwa.bwa_idx_build(
                 fasta_bytes, prefix_bytes, self.algo_type, self.block_size
@@ -1035,49 +1029,54 @@ class BwaIndexer(object):
 
     def _parse_progress_line(self, line):
         """Parse a progress line from BWA stderr output."""
-        import re
-        
         # Store all messages
         self.progress["messages"].append(line)
-        
+
         # Parse text length
-        match = re.search(r'\[BWTIncCreate\] textLength=(\d+)', line)
+        match = re.search(r"\[BWTIncCreate\] textLength=(\d+)", line)
         if match:
             self.progress["text_length"] = int(match.group(1))
             return
-        
+
         # Parse iteration progress
-        match = re.search(r'\[BWTIncConstructFromPacked\] (\d+) iterations done\. (\d+) characters processed', line)
+        match = re.search(
+            r"\[BWTIncConstructFromPacked\] (\d+) iterations done\. (\d+) characters processed",
+            line,
+        )
         if match:
             self.progress["iterations"] = int(match.group(1))
             self.progress["characters_processed"] = int(match.group(2))
             return
-        
+
         # Parse completion
-        if 'Finished constructing BWT' in line:
-            match = re.search(r'(\d+) iterations', line)
+        if "Finished constructing BWT" in line:
+            match = re.search(r"(\d+) iterations", line)
             if match:
                 self.progress["iterations"] = int(match.group(1))
-    
+
     def get_progress(self):
         """Get current indexing progress.
-        
+
         :returns: Dictionary with progress information
         """
         return self.progress.copy()
-    
+
     @property
     def progress_percent(self):
         """Get progress as percentage (if text_length is known).
-        
+
         :returns: Float percentage (0-100) or None if not available
         """
-        if self.progress["text_length"] > 0 and self.progress["characters_processed"] > 0:
-            return (self.progress["characters_processed"] / self.progress["text_length"]) * 100
+        if (
+            self.progress["text_length"] > 0
+            and self.progress["characters_processed"] > 0
+        ):
+            return (
+                self.progress["characters_processed"] / self.progress["text_length"]
+            ) * 100
         return None
 
 
-
 def get_parser():
     parser = argparse.ArgumentParser("Align a sequence with bwa mem.")
     parser.add_argument("index", help="bwa index base path.")
